/*
    cve-manager : CVE management tool
    Copyright (C) 2017-2022 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include    "data_preproc.h"
#include    "data_preproc_nvd.h"
#include    "data_preproc_fstec.h"

//-----------------------------------------------------------------------------
// Replace or get rid of some of the symbols

bool ManageForbiddenSymbols(tree::TSelectedData &selected_data, string &err){

    for(auto &entry : selected_data){
        for(auto &item : entry){
            for(auto &str : item){
                // Replacing double quotation marks with single quotation marks
                replace(str.begin(), str.end(), '\"', '\'');
                // Removing backslashes at the end
                while(!str.empty() && str.back() == '\\'){
                    str.erase(str.end() - 1);
                    }
                }
            }
        }

    return true;
    }

//-----------------------------------------------------------------------------
// Call a preprocess-function that corresponds to a given target

bool Preprocess(const string &target, tree::TSelectedData &selected_data,
        const map<string, TExcludedProducts> &excluded_products, string &err){

    printer.LineBegin("Preprocessing " + ToUpper(target) + " selected data");

    string msg;

    const auto cpe_excl   = excluded_products.find(CPE_EXCL);
    const auto fstec_excl = excluded_products.find(FSTEC_EXCL);

    const bool success =
        (target != TARGET_CPE   ||
            PreprocessCPEDict(selected_data, cpe_excl->second, msg)) &&
        (target != TARGET_NVD   ||
            PreprocessNVDVul(selected_data, cpe_excl->second, msg)) &&
        (target != TARGET_FSTEC ||
            PreprocessFSTECVul(selected_data, fstec_excl->second, msg)) &&
        ManageForbiddenSymbols(selected_data, msg);

    if(success){
        if(!msg.empty()){
            printer.LineEnd("[NOTE: " + msg + "]");
            }
        else{
            printer.Success();
            }
        }
    else{
        err = msg;
        printer.Err(err);
        }

    return success;
    }

//-----------------------------------------------------------------------------
