/*
    cve-manager : CVE management tool
    Copyright (C) 2017-2022 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     cve_import_databaseH
#define     cve_import_databaseH

#include    "libcve-manager/src/database.h"
#include    "main.h"
#include    "lists.h"

//-----------------------------------------------------------------------------
// MySQL DB interface

class TSpecificDBInterface : public TDatabase{

    private:

        bool appending = false; // Insertions should be performed without
                                //  deleting everything from tables beforehand

        // Delete the DB and create an empty one with the same name
        bool ClearDB(void);
        // Create table with given name and structure described in the query
        bool CreateTable(const string &table, const string &query,
                vector<string> child_tables={});
        // Create specified tables
        bool RecreateChildTables(const vector<string> &tables);
        // Create table for CPE dictionary
        bool CreateCPETable(void);
        // Create CVE table for the given target (FSTEC, NVD, ...)
        bool CreateCVETable(const vector<TBranch> &branches,
                const string &target);
        // Create branch table for the given target (src or bin)
        bool CreateBranchTable(const string &branch_name, const string &target);
        // Create target table (src and bin) for each given branches
        bool CreateBranchTables(const vector<TBranch> &branches,
                const string &target);
        // Form CSV import query
        string FormCSVImportQuery(const string &csv_path,
                const string &table) const;

    public:

        // Create DB structure
        bool CreateFullStruct(const vector<TBranch> &branches,
                const vector<string> &excluded_vulsrc);
        // Call table-creation function for specified target
        bool CreateTargetTable(const vector<TBranch> &branches,
                const string &target, bool nested=false);
        // Import list of package descriptions into the DB
        bool ImportPackages(TBranch &branch);
        // Import data in CSV format into the DB
        bool ImportCSV(const vector<TBranch> &branches, const string &target,
                const string &csv_path, bool force_clean=false);
    };

//-----------------------------------------------------------------------------

#endif
