/*
    cve-manager : CVE management tool
    Copyright (C) 2017-2022 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     cve_import_rpmH
#define     cve_import_rpmH

#include    <map>

#include    "main.h"
#include    "lists.h"

//-----------------------------------------------------------------------------

typedef set<string> TBinPackages;
typedef map<string, TBinPackages> TPackages;

const map<string, int> vul_markers = {
    {"cve", PACK_FIXED_CVE},
    {"bdu", PACK_FIXED_BDU}
    };
const set<char> spec_symbols = {'\''};

//-----------------------------------------------------------------------------

pair<TPackages, string> GetSysPackages(void);
pair<map<string, int>, set<string>> GetMetadata(
    const string &srpms_dir,
    const vector<string> &rpms_dirs,
    map<string, map<int, string>> &src,
    const map<string, set<string>> &bin,
    const bool is_local_sys,
    const set<string> &excluded_src,
    vector<string> &warn);

//-----------------------------------------------------------------------------

class TRpmInterface{

    public:

        function<pair<set<string>, string>(const string&)> PostProcess;
        function<pair<int, string>(const string&)> RequestRPM;
        function<void(map<int, string>&, const set<string>&)> AppendData;

        TRpmInterface(const string &target="", const bool is_local_sys=false);
        ~TRpmInterface(void){};
    };

//-----------------------------------------------------------------------------

#endif
