/*
    cve-manager : CVE management tool
    Copyright (C) 2017-2022 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     cve_import_treeH
#define     cve_import_treeH

#include    "main.h"

#ifdef      CVEIMPORT_DEBUGGING
#include    "libtree/src/tree.h"
#include    "libtree/src/xml.h"
#include    "libtree/src/json.h"
#else
#include    <libtree/tree.h>
#include    <libtree/xml.h>
#include    <libtree/json.h>
#endif

#define     VUL_YEAR                        0   // Both FSTEC & NVD identifiers got a year part
#define     VUL_NUM                         1   // Both FSTEC & NVD identifiers got a number part
#define     VUL_ID                          VUL_NUM // A complete id during a parsing, a number part after preprocessing

#define     FSTEC_VUL_Q                     23  // Number of named (marked) branches of the FSTEC structure tree
#define     FSTEC_VUL_YEAR                  VUL_YEAR // -
#define     FSTEC_VUL_ID                    VUL_NUM  // vulnerabilities > vul > identifier >> text
#define     FSTEC_VUL_NAME                  2   // vulnerabilities > vul > name >> text
#define     FSTEC_VUL_DESCRIPTION           3   // vulnerabilities > vul > description >> text
#define     FSTEC_VUL_SOFT_VENDOR           4   // vulnerabilities > vul > vulnerable_software >> soft >> vendor
#define     FSTEC_VUL_SOFT_NAME             5   // vulnerabilities > vul > vulnerable_software >> soft >> name
#define     FSTEC_VUL_SOFT_VER              6   // vulnerabilities > vul > vulnerable_software >> soft >> version
#define     FSTEC_VUL_SOFT_TYPE             7   // vulnerabilities > vul > vulnerable_software >> soft >> types >> type
#define     FSTEC_VUL_ENV_VENDOR            8   // vulnerabilities > vul > environment >> os >> vendor >> text
#define     FSTEC_VUL_ENV_NAME              9   // vulnerabilities > vul > environment >> os >> name >> text
#define     FSTEC_VUL_ENV_VER               10  // vulnerabilities > vul > environment >> os >> version >> text
#define     FSTEC_VUL_ENV_PLATFORM          11  // vulnerabilities > vul > environment >> os >> platform >> text
#define     FSTEC_VUL_CWE                   12  // vulnerabilities > vul > cwe >> identifier >> text
#define     FSTEC_VUL_IDENTIFY_DATE         13  // vulnerabilities > vul > identify_date >> text
#define     FSTEC_VUL_CVSS2_SCORE           14  // vulnerabilities > vul > cvss2 >> vector >> Attribute "score"
#define     FSTEC_VUL_CVSS3_SCORE           15  // vulnerabilities > vul > cvss2 >> vector >> Attribute "score"
#define     FSTEC_VUL_SEVERITY              16  // vulnerabilities > vul > severity >> text
#define     FSTEC_VUL_SOLUTION              17  // vulnerabilities > vul > solution >> text
#define     FSTEC_VUL_STATUS                18  // vulnerabilities > vul > vul_status >> text
#define     FSTEC_VUL_EXPLOIT_STATUS        19  // vulnerabilities > vul > exploit_status >> text
#define     FSTEC_VUL_FIX_STATUS            20  // vulnerabilities > vul > fix_status >> text
#define     FSTEC_VUL_SOURCES               21  // vulnerabilities > vul > sources >> text
#define     FSTEC_VUL_OTHER_IDS             22  // vulnerabilities > vul > identifiers > identifier >> text

#define     FSTEC_VUL_COMP_Q                17  // Number of named (marked) branches of the FSTEC structure tree
#define     FSTEC_VUL_COMP_YEAR             VUL_YEAR // -
#define     FSTEC_VUL_COMP_ID               VUL_NUM  // vulnerabilities > vul > identifier >> text
#define     FSTEC_VUL_COMP_NAME             2   // vulnerabilities > vul > name >> text
#define     FSTEC_VUL_COMP_DESCRIPTION      3   // vulnerabilities > vul > description >> text
#define     FSTEC_VUL_COMP_SOFTWARE_LIST    4   // vulnerabilities > vul > vulnerable_software >> soft >> vendor
#define     FSTEC_VUL_COMP_ENV              5   // vulnerabilities > vul > environment >> os >> name >> text
#define     FSTEC_VUL_COMP_CWE              6   // vulnerabilities > vul > cwe >> identifier >> text
#define     FSTEC_VUL_COMP_IDENTIFY_DATE    7   // vulnerabilities > vul > identify_date >> text
#define     FSTEC_VUL_COMP_CVSS2_SCORE      8   // vulnerabilities > vul > cvss >> vector >> Attribute "score"
#define     FSTEC_VUL_COMP_CVSS3_SCORE      9   // vulnerabilities > vul > cvss3 >> vector >> Attribute "score"
#define     FSTEC_VUL_COMP_SEVERITY         10   // vulnerabilities > vul > severity >> text
#define     FSTEC_VUL_COMP_SOLUTION         11  // vulnerabilities > vul > solution >> text
#define     FSTEC_VUL_COMP_STATUS           12  // vulnerabilities > vul > vul_status >> text
#define     FSTEC_VUL_COMP_EXPLOIT_STATUS   13  // vulnerabilities > vul > exploit_status >> text
#define     FSTEC_VUL_COMP_FIX_STATUS       14  // vulnerabilities > vul > fix_status >> text
#define     FSTEC_VUL_COMP_SOURCES          15  // vulnerabilities > vul > sources >> text
#define     FSTEC_VUL_COMP_CVE_IDS          16  // vulnerabilities > vul > identifiers > identifier >> text

#define     NVD_CVE_Q                       18  // Number of needed nodes of NVD CVE JSON doc
#define     NVD_CVE_YEAR                    VUL_YEAR // -
#define     NVD_CVE_ID                      VUL_NUM  // CVE_Items > cve > CVE_data_meta > CVE_data_meta > ID
#define     NVD_CVE_SOFTWARE                2   // CVE_Items > configurations > nodes > ?children > cpe_match > cpe23Uri
#define     NVD_CVE_PUBLISHED               3   // CVE_Items > publishedDate
#define     NVD_CVE_MODIFIED                4   // CVE_Items > lastModifiedDate
#define     NVD_CVE_CVSS_SCORE_V2           5   // CVE_Items > impact > baseMetricV2 > cvssV2 > baseScore
#define     NVD_CVE_CVSS_SCORE_V3           6   // CVE_Items > impact > baseMetricV3 > cvssV3 > baseScore
#define     NVD_CVE_CWE_ID                  7   // CVE_Items > cve > problemtype > problemtype_data > description > value
#define     NVD_CVE_REFS                    8   // CVE_Items > cve > references > reference_data > url
#define     NVD_CVE_REFS_TAG                9   // CVE_Items > cve > references > reference_data > tags [SLAVE BRANCH]
#define     NVD_CVE_SUMMARY                 10  // CVE_Items > cve > description > description_data > value
#define     NVD_CVE_SOFTWARE_LOGIC1         11  // CVE_Items > configurations > nodes > operator
#define     NVD_CVE_SOFTWARE_LOGIC2         12  // CVE_Items > configurations > nodes > children > operator
#define     NVD_CVE_SOFTWARE_VULNERABLE     13  // CVE_Items > configurations > nodes > ?children > cpe_match > vulnerable [SLAVE BRANCH]
#define     NVD_CVE_SOFTWARE_START_EXCL     14  // CVE_Items > configurations > nodes > ?children > cpe_match > versionStartExcluding [SLAVE BRANCH]
#define     NVD_CVE_SOFTWARE_START_INCL     15  // CVE_Items > configurations > nodes > ?children > cpe_match > versionStartIncluding [SLAVE BRANCH]
#define     NVD_CVE_SOFTWARE_END_EXCL       16  // CVE_Items > configurations > nodes > ?children > cpe_match > versionEndExcluding [SLAVE BRANCH]
#define     NVD_CVE_SOFTWARE_END_INCL       17  // CVE_Items > configurations > nodes > ?children > cpe_match > versionEndIncluding [SLAVE BRANCH]

#define     NVD_CVE_PATCH                   NVD_CVE_REFS_TAG // Patch field for a string of patches extracted from refs

#define     CPE_Q                           3   // Number of named (marked) branches of the CPE structure tree
#define     CPE_FULLNAME                    0   // cpe-list > cpe-23:cpe23-item > Attribute "name"
#define     CPE_TITLE                       1   // cpe-list > cpe-item > title > text >> content
#define     CPE_REFS                        2   // cpe-list > cpe-item > references > reference >> Attribute "href"

#define     CPE_SEP_Q                       5   // Number of colums in CPE separate representation table (6th param not for import)
#define     CPE_SEP_REFS                    0   // CPE references
#define     CPE_SEP_PART                    1   // CPE part component
#define     CPE_SEP_VENDOR                  2   // CPE vendor component
#define     CPE_SEP_PRODUCT                 3   // CPE product component
#define     CPE_SEP_NON_FREE                4   // 1 - there are no titles of this product that state that this app is for non-free OS
                                                // 2 - all titles (...)
                                                // 3 - some titles (...)
#define     CPE_SEP_VER                     5   // CPE version component (not for import)
#define     CPE_SEP_REL                     6   // CPE release component (not for import)
#define     CPE_SEP_AFTER_REL               7   // CPE component that follows release component (not for import)

#define     SEL_L_BORDER                    ""  // Left  border of an item of selected data
#define     SEL_R_BORDER                    ""  // Right border of an item of selected data
#define     SEL_DELIM                       " " // Delimiter between items of selected data

//-----------------------------------------------------------------------------

// Form base nodes to select data from JSON document
pair<string, vector<tree::TBranchDraft>> FormXmlBranchDrafts(
    const string &target, string &err);
// Form branch drafts to select data from JSON document
pair<string, vector<tree::TBranchDraft>> FormJsonBranchDrafts(
    const string &target, string &err);

//-----------------------------------------------------------------------------

#endif
